@@ -240,13 +240,23 @@ plot3drequest()
 	strcpy(set_dummy_var[1], "v");
     }
 
-    /* put stuff into arrays to simplify access */
-    AXIS_INIT3D(FIRST_X_AXIS, 0, 0);
-    AXIS_INIT3D(FIRST_Y_AXIS, 0, 0);
-    AXIS_INIT3D(FIRST_Z_AXIS, 0, 1);
-    AXIS_INIT3D(U_AXIS, 1, 0);
-    AXIS_INIT3D(V_AXIS, 1, 0);
-    AXIS_INIT3D(COLOR_AXIS, 0, 1);
+    /* initialize the arrays from the 'set' scalars */
+    axis_init(&axis_array[FIRST_X_AXIS], FALSE);
+    axis_init(&axis_array[FIRST_Y_AXIS], FALSE);
+    axis_init(&axis_array[FIRST_Z_AXIS], TRUE);
+    axis_init(&axis_array[U_AXIS], FALSE);
+    axis_init(&axis_array[V_AXIS], FALSE);
+    axis_init(&axis_array[COLOR_AXIS], TRUE);
+
+    /* Always be prepared to restore the autoscaled values on "refresh"
+     * Dima Kogan April 2018
+     * FIXME: Could we fold this into axis_init?
+     */
+    for (axis = 0; axis < NUMBER_OF_MAIN_VISIBLE_AXES; axis++) {
+	AXIS *this_axis = &axis_array[axis];
+	if (this_axis->set_autoscale != AUTOSCALE_NONE)
+	    this_axis->range_flags |= RANGE_WRITEBACK;
+    }
 
 #ifdef NONLINEAR_AXES
     /* Nonlinear mapping of x or y via linkage to a hidden primary axis. */
@@ -347,44 +357,55 @@ refresh_3dbounds(struct surface_points *first_plot, int nplots)
 	    continue;
 	}
 
-	for ( this_curve = this_plot->iso_crvs; this_curve; this_curve = this_curve->next) {
+	for (this_curve = this_plot->iso_crvs; this_curve; this_curve = this_curve->next) {
 
-	for (i=0; i<this_curve->p_count; i++) {
-	    struct coordinate GPHUGE *point = &this_curve->points[i];
-
-	    if (point->type == UNDEFINED)
-		continue;
+	    /* VECTOR plots consume two iso_crvs structures, one for heads and one for tails.
+	     * Only the first one has the true point count; the second one claims zero.
+	     * FIXME: Maybe we should change this?
+	     */
+	    int n_points;
+	    if (this_plot->plot_style == VECTOR)
+		n_points = this_plot->iso_crvs->p_count;
 	    else
-		point->type = INRANGE;
+		n_points = this_curve->p_count;
 
-	    /* If the state has been set to autoscale since the last plot,
-	     * mark everything INRANGE and re-evaluate the axis limits now.
-	     * Otherwise test INRANGE/OUTRANGE against previous axis limits.
-	     */
+	    for (i=0; i<n_points; i++) {
+		struct coordinate *point = &this_curve->points[i];
 
-	    /* This autoscaling logic is parallel to that in
-	     * refresh_bounds() in plot2d.c
-	     */
-	    if (!this_plot->noautoscale) {
-		autoscale_one_point(x_axis, point->x);
-		autoscale_one_point(y_axis, point->y);
-	    }
-	    if (!inrange(point->x, x_axis->min, x_axis->max)
-	    ||  !inrange(point->y, y_axis->min, y_axis->max)) {
-		point->type = OUTRANGE;
-		continue;
-	    }
-	    if (!this_plot->noautoscale) {
-		autoscale_one_point(z_axis, point->z);
-	    }
-	    if (!inrange(point->z, z_axis->min, z_axis->max)) {
-		point->type = OUTRANGE;
-		continue;
-	    }
-	}	/* End of this curve */
-	}
+		if (point->type == UNDEFINED)
+		    continue;
+		else
+		    point->type = INRANGE;
 
-    }	/* End of this plot */
+		/* If the state has been set to autoscale since the last plot,
+		 * mark everything INRANGE and re-evaluate the axis limits now.
+		 * Otherwise test INRANGE/OUTRANGE against previous axis limits.
+		 */
+
+		/* This autoscaling logic is parallel to that in
+		 * refresh_bounds() in plot2d.c
+		 */
+		if (!this_plot->noautoscale) {
+		    autoscale_one_point(x_axis, point->x);
+		    autoscale_one_point(y_axis, point->y);
+		}
+		if (!inrange(point->x, x_axis->min, x_axis->max)
+		||  !inrange(point->y, y_axis->min, y_axis->max)) {
+		    point->type = OUTRANGE;
+		    continue;
+		}
+		if (!this_plot->noautoscale) {
+		    autoscale_one_point(z_axis, point->z);
+		}
+		if (!inrange(point->z, z_axis->min, z_axis->max)) {
+		    point->type = OUTRANGE;
+		    continue;
+		}
+
+	    }	/* End of points in this curve */
+	}   /* End of curves in this plot */
+
+    }	/* End of plots in this splot command */
 
     /* handle 'reverse' ranges */
     axis_revert_range(FIRST_X_AXIS);
@@ -1115,8 +1136,11 @@ get_3ddata(struct surface_points *this_plot)
 
 	    /* At this point we have stored the point coordinates. Now we need to copy */
 	    /* x,y,z into the text_label structure and add the actual text string.     */
-	    if (this_plot->plot_style == LABELPOINTS)
-		store_label(this_plot->labels, cp, xdatum, df_tokens[3], color);
+	    if (this_plot->plot_style == LABELPOINTS) {
+		/* Aug 2018: only store INRANGE labels. This is a CHANGE. */
+		if (cp->type == INRANGE)
+		    store_label(this_plot->labels, cp, xdatum, df_tokens[3], color);
+	    }
 
 	    if (this_plot->plot_style == RGBIMAGE || this_plot->plot_style == RGBA_IMAGE) {
 		/* We will autoscale the RGB components to  a total range [0:255]
@@ -1724,6 +1748,11 @@ eval_3dplots()
 			    set_labelstyle = TRUE;
 			    continue;
 			}
+		    } else if (this_plot->lp_properties.p_type == PT_CHARACTER) {
+			if  (equals(c_token, ","))
+			    break;
+			else
+			    continue;
 		    }
 		}
 
@@ -2233,7 +2262,7 @@ eval_3dplots()
 	update_primary_axis_range(&axis_array[FIRST_Z_AXIS]);
 	extend_primary_ticrange(&axis_array[FIRST_Z_AXIS]);
     } else {
-	axis_checked_extend_empty_range(FIRST_Z_AXIS, splot_map ? NULL : "All points y value undefined");
+	axis_checked_extend_empty_range(FIRST_Z_AXIS, splot_map ? NULL : "All points z value undefined");
 	axis_revert_and_unlog_range(FIRST_Z_AXIS);
     }
 
