@@ -495,6 +495,14 @@ term_initialise()
 	FPRINTF((stderr, "- calling term->init()\n"));
 	(*term->init) ();
 	term_initialised = TRUE;
+#ifdef HAVE_LOCALE_H
+	/* This is here only from an abundance of caution (a.k.a. paranoia).
+	 * Some terminals (wxt qt caca) are known to change the locale when
+	 * initialized.  Others have been implicated (gd).  Rather than trying
+	 * to catch all such offenders one by one, cover for all of them here.
+	 */
+	setlocale(LC_NUMERIC, "C");
+#endif
     }
 }
 
@@ -2641,11 +2649,17 @@ estimate_plaintext(char *enhancedtext)
 void
 ignore_enhanced(TBOOLEAN flag)
 {
-    /* Force a return to the default font */
+#if (0)
+    /* Apr 2018: This code was introduced long ago (2005; Bug #266) to address
+     * a glitch in the postscript terminal that left the last-used font in
+     * an enhanced text string active afterwards.
+     * We now deal with this in ENHPS_put_text() instead.
+     */
     if (flag && !ignore_enhanced_text) {
 	ignore_enhanced_text = TRUE;
 	term->set_font("");
     }
+#endif
     ignore_enhanced_text = flag;
 }
 
