@@ -1,7 +1,3 @@
-#ifndef lint
-static char *RCSid() { return RCSid("$Id: unset.c,v 1.254.2.1 2017/07/08 00:11:36 sfeam Exp $"); }
-#endif
-
 /* GNUPLOT - unset.c */
 
 /*[
@@ -45,6 +41,7 @@ static char *RCSid() { return RCSid("$Id: unset.c,v 1.254.2.1 2017/07/08 00:11:3
 #include "hidden3d.h"
 #include "jitter.h"
 #include "misc.h"
+#include "multiplot.h"
 #include "parse.h"
 #include "plot.h"
 #include "plot2d.h"
@@ -407,6 +404,9 @@ unset_command()
 	    c_token++;
 	}
 	break;
+    case S_RGBMAX:
+	rgbmax = 255;
+	break;
     case S_SAMPLES:
 	unset_samples();
 	break;
@@ -812,6 +812,8 @@ unset_cntrparam()
     contour_order = DEFAULT_CONTOUR_ORDER;
     contour_levels = DEFAULT_CONTOUR_LEVELS;
     contour_levels_kind = LEVELS_AUTO;
+    contour_firstlinetype = 0;
+    contour_sortlevels = FALSE;
 }
 
 /* process 'unset cntrlabel' command */
@@ -941,6 +943,7 @@ unset_grid()
 	axis_array[i].gridminor = FALSE;
     }
     polar_grid_angle = 0;
+    grid_vertical_lines = FALSE;
 }
 
 
@@ -1433,7 +1436,6 @@ unset_pm3d()
     /* reset styles, required to 'plot something' after e.g. 'set pm3d map' */
     if (data_style == PM3DSURFACE) data_style = POINTSTYLE;
     if (func_style == PM3DSURFACE) func_style = LINES;
-    pm3d.border.l_type = LT_NODRAW;
 }
 
 
@@ -1962,6 +1964,7 @@ reset_command()
 
     unset_size();
     aspect_ratio = 0.0;		/* don't force it */
+    rgbmax = 255;
 
     unset_origin();
     unset_timestamp();
@@ -1995,6 +1998,10 @@ reset_command()
     mouse_setting = default_mouse_setting;
 #endif
 
+    /* restore previous multiplot offset and margins */
+    if (multiplot)
+	multiplot_reset();
+
     unset_missing();
     free(df_separators);
     df_separators = NULL;
